""" This module stores custom tokenizers. """
import re


def white_space(data):
    return data.split()


def nltk(data):
    try:
        from nltk.tokenize import word_tokenize
        return word_tokenize(data)
    except ImportError:
        raise ImportError('nltk is missing, please run `pip install nltk`')



import sys
import argparse
import csv

from cyzil import tokenizers


def _get_tokenizer(option=None):
    if (option is None) or (option == 'space'):
        return tokenizers.white_space
    elif option == 'nltk':
        return tokenizers.nltk


def load_data(path, tokenizer_option):
    with open(path, mode='rt', encoding='utf-8') as f:
        data = f.read().strip().split('\n')
    tokenizer = _get_tokenizer(tokenizer_option)
    tokenized = [tokenizer(sentence) for sentence in data]
    return tokenized


def store_output(data, path_file):
    # append csv extension if a path doesn't have one
    if path_file.find('.csv') == -1:
        path_file += '.csv'
    with open(path_file, mode='w', encoding="utf-8", newline='\n') as f:
        writer = csv.writer(f, delimiter=',')
        for row in data:
            writer.writerow(row)

def check_help(docs):
    if (len(sys.argv) == 2) and \
       ((sys.argv[1] == '--help') or (sys.argv[1] == '-h')):
       parser = argparse.ArgumentParser(description=docs,
                                        formatter_class=argparse.RawDescriptionHelpFormatter)
       parser.parse_args()
       sys.exit(1)
    elif len(sys.argv) == 2:
       print('Illegal argument. Please refere to help by --help or -h')
       sys.exit(1)



"""
A module for cyzil cli tool.
"""
from .bleu_cli import bleu_corpus, bleu_points
from .edit_distance_cli import edit_distance_corpus, edit_distance_points


__all__ = [
    "bleu_corpus",
    "bleu_points",
    "edit_distance_corpus",
    "edit_distance_points"
]



"""
Cyzil provides tools that enable quick and in-depth analysis of
sequence generation models such as machine translation models. It contains
a Cython module that provides fast computation of standard metrics such as
BLEU score.
"""

from bleu import bleu_sentence, bleu_corpus, bleu_points
from edit_distance import edit_distance_sentence, edit_distance_corpus, edit_distance_points

__all__ = [
    "bleu_sentence",
    "bleu_corpus",
    "bleu_points",
    "edit_distance_sentence",
    "edit_distance_corpus",
    "edit_distance_points"
]



import argparse
import sys

from .utils import load_data, store_output

import cyzil


def _parse_argument(docs):
    parser = argparse.ArgumentParser(description=docs)
    parser.add_argument(
        "--reference",
        required=True,
        help="path to a reference file, where each serntence is in a separate line"
    )
    parser.add_argument(
        "--candidate",
        required=True,
        help="path to a candiate file, where each serntence is in a separate line"
    )
    parser.add_argument(
        "--ngram",
        default=4,
        help="the maximum order of ngram to compute the score"
    )
    parser.add_argument(
        "--tokenizer",
        default='space',
        choices=['space', 'nltk'],
        help="a way to tokenize sentences (white space by default)"
    )
    parser.add_argument(
        "-o",
        "--output",
        help="a file path to store output in csv format"
    )
    return parser.parse_args()


def bleu_corpus():
    parser = _parse_argument("compute corpus-level BLEU score")
    reference = load_data(parser.reference, parser.tokenizer)
    candidate = load_data(parser.candidate, parser.tokenizer)
    scores = cyzil.bleu_corpus(reference, candidate, parser.ngram)
    print(scores)


def bleu_points():
    parser = _parse_argument("compute BLEU score for each translation pair in corpus")
    reference = load_data(parser.reference, parser.tokenizer)
    candidate = load_data(parser.candidate, parser.tokenizer)
    scores = cyzil.bleu_points(reference, candidate, parser.ngram)
    if parser.output is not None:
        store_output(scores, parser.output)
    else:
        print(scores)
